#pragma once

#define WIN32_LEAN_AND_MEAN

#include <wchar.h>
#include <iostream>
#include <windows.h>
#include <winioctl.h>
#include "Resource.h"
using namespace std;

const int MAX_PARTITIONS=40;

typedef LONG NTSTATUS, *PNTSTATUS;

#define NT_SUCCESS(StatCode)  ((NTSTATUS)(StatCode) == 0)
#define SECTORSIZE 512

#pragma pack(push,1)

typedef struct _FAT32_BOOT_SECTOR
{
  unsigned char  magic0;                      // 0
  unsigned char  res0;                        // 1
  unsigned char  magic1;                      // 2
  unsigned char  OEMName[8];                  // 3
  unsigned short BytesPerSector;              // 11
  unsigned char  SectorsPerCluster;           // 13
  unsigned short ReservedSectors;             // 14
  unsigned char  FATCount;                    // 16
  unsigned short RootEntries;                 // 17
  unsigned short Sectors;                     // 19
  unsigned char  Media;                       // 21
  unsigned short FATSectors;                  // 22
  unsigned short SectorsPerTrack;             // 24
  unsigned short Heads;                       // 26
  unsigned long  HiddenSectors;               // 28
  unsigned long  SectorsHuge;                 // 32
  unsigned long  FATSectors32;                // 36
  unsigned short ExtFlag;                     // 40
  unsigned short FSVersion;                   // 42
  unsigned long  RootCluster;                 // 44
  unsigned short FSInfoSector;                // 48
  unsigned short BootBackup;                  // 50
  unsigned char  Res3[12];                    // 52
  unsigned char  Drive;                       // 64
  unsigned char  Res4;                        // 65
  unsigned char  ExtBootSignature;            // 66
  unsigned long  VolumeID;                    // 67
  unsigned char  VolumeLabel[11];             // 71
  unsigned char  SysType[8];                  // 82
  unsigned char  Res2[420];                   // 90
  unsigned short  Signature1;                  // 510
} FAT32_BOOT_SECTOR, *PFAT32_BOOT_SECTOR;

#pragma pack(pop)
#pragma pack(push,1)

typedef struct _FAT32_FSINFO
{
  unsigned long  LeadSig;          // 0
  unsigned char  Res1[480];        // 4
  unsigned long  StrucSig;         // 484
  unsigned long  FreeCount;        // 488
  unsigned long  NextFree;         // 492
  unsigned long  Res2[3];          // 496
  unsigned long  TrailSig;         // 508
} FAT32_FSINFO, *PFAT32_FSINFO;

#pragma pack(pop)
#pragma pack(push,1)

typedef struct DirEntry_t {
	BYTE    name[11];            /* file name */
	BYTE    attribute;          /* file attribute
								bit 4    directory flag
								bit 3    volume flag
								bit 2    hidden flag
								bit 1    system flag
								bit 0    read only flag */
	BYTE    reserved;           /* use NT or same OS */
	BYTE    createTimeMs;       /* VFAT Ŏgpt@C쐬10~b (0 ` 199) */
	BYTE    createTime[2];      /* VFAT Ŏgpt@C쐬 */
	BYTE    createDate[2];      /* VFAT Ŏgpt@C쐬t */
	BYTE    accessDate[2];      /* VFAT Ŏgpt@CEANZXt */
	BYTE    clusterHighWord[2]; /* FAT32 ŎgpNX^ԍ̏ 16 bits */
	BYTE    updateTime[2];
	BYTE    updateDate[2];
	BYTE    cluster[2];         /* start cluster number */
	BYTE    fileSize[4];        /* file size in bytes (directory is always zero) */
}   DirEntry;

#pragma pack(pop)

class Fat32Format
{
protected:
	HANDLE m_hPhysicalDisk;
	HWND m_hParentDlg;
	LONGLONG m_llVolumeSize;
public:
	Fat32Format(void);
	~Fat32Format(void);
	bool Initialize(CString cstrDeviceName);
	int SetPartition(int nIndex, DWORD size);
	int DeletePartition(CString cstrDrive, int nIndex);
	int FormatPartition(CString cstrDrive, CString cstrVolumeLabel,int nIndex, BOOL bQuickFormat, CProgressCtrl *pProgFormat, HWND hParentDlg);
	int InitDrive();
protected:
	void ErrorMessage(DWORD);
	int Fat32WriteBootSector(HANDLE FileHandle, PFAT32_BOOT_SECTOR BootSector);
	int Fat32WriteFsInfo(HANDLE FileHandle, PFAT32_BOOT_SECTOR BootSector);
	int Fat32WriteFAT(HANDLE FileHandle, ULONG SectorOffset, PFAT32_BOOT_SECTOR BootSector);
	int Fat32WriteRootDirectory(HANDLE FileHandle, PFAT32_BOOT_SECTOR BootSector);
	int Fat32WriteDataSector(HANDLE FileHandle, PFAT32_BOOT_SECTOR BootSector, CProgressCtrl *pProgFormat);
	int StartFormat(HANDLE  FileHandle,
				CString cstrVolumeLabel,
				PPARTITION_INFORMATION  PartitionInfo,
				PDISK_GEOMETRY DiskGeometry,
				BOOL  QuickFormat,
				DWORD  ClusterSize,
				CProgressCtrl *pProgFormat);
};
